@@ -5,8 +5,10 @@ multidict. It behaves mostly like a dict but it can have
 several values for the same key.
 """
 
+from typing import TYPE_CHECKING
+
 from ._abc import MultiMapping, MutableMultiMapping
-from ._compat import USE_CYTHON_EXTENSIONS
+from ._compat import USE_EXTENSIONS
 
 __all__ = (
     "MultiMapping",
@@ -17,31 +19,29 @@ __all__ = (
     "CIMultiDict",
     "upstr",
     "istr",
-    "getversion"
+    "getversion",
 )
 
-__version__ = "4.7.3"
+__version__ = "6.2.0"
 
 
-try:
-    if not USE_CYTHON_EXTENSIONS:
-        raise ImportError
-    from ._multidict import (
-        MultiDictProxy,
-        CIMultiDictProxy,
-        MultiDict,
-        CIMultiDict,
-        istr,
-        getversion,
-    )
-except ImportError:  # pragma: no cover
+if TYPE_CHECKING or not USE_EXTENSIONS:
     from ._multidict_py import (
-        MultiDictProxy,
+        CIMultiDict,
         CIMultiDictProxy,
         MultiDict,
-        CIMultiDict,
-        istr,
+        MultiDictProxy,
         getversion,
+        istr,
+    )
+else:
+    from ._multidict import (
+        CIMultiDict,
+        CIMultiDictProxy,
+        MultiDict,
+        MultiDictProxy,
+        getversion,
+        istr,
     )
 
 
