@@ -1,6 +1,8 @@
 #include "Python.h"
 #include "structmember.h"
 
+#include "_multilib/pythoncapi_compat.h"
+
 // Include order important
 #include "_multilib/defs.h"
 #include "_multilib/istr.h"
@@ -9,22 +11,23 @@
 #include "_multilib/iter.h"
 #include "_multilib/views.h"
 
+#if PY_MINOR_VERSION < 12
 #ifndef _PyArg_UnpackKeywords
 #define FASTCALL_OLD
 #endif
+#endif
 
 
 static PyObject *collections_abc_mapping;
 static PyObject *collections_abc_mut_mapping;
 static PyObject *collections_abc_mut_multi_mapping;
+static PyObject *repr_func;
 
 static PyTypeObject multidict_type;
 static PyTypeObject cimultidict_type;
 static PyTypeObject multidict_proxy_type;
 static PyTypeObject cimultidict_proxy_type;
 
-static PyObject *repr_func;
-
 #define MultiDict_CheckExact(o) (Py_TYPE(o) == &multidict_type)
 #define CIMultiDict_CheckExact(o) (Py_TYPE(o) == &cimultidict_type)
 #define MultiDictProxy_CheckExact(o) (Py_TYPE(o) == &multidict_proxy_type)
@@ -153,13 +156,17 @@ _multidict_append_items_seq(MultiDictObject *self, PyObject *arg,
             Py_INCREF(value);
         }
         else if (PyList_CheckExact(item)) {
-            if (PyList_GET_SIZE(item) != 2) {
+            if (PyList_Size(item) != 2) {
+                goto invalid_type;
+            }
+            key = PyList_GetItemRef(item, 0);
+            if (key == NULL) {
+                goto invalid_type;
+            }
+            value = PyList_GetItemRef(item, 1);
+            if (value == NULL) {
                 goto invalid_type;
             }
-            key = PyList_GET_ITEM(item, 0);
-            Py_INCREF(key);
-            value = PyList_GET_ITEM(item, 1);
-            Py_INCREF(value);
         }
         else if (PySequence_Check(item)) {
             if (PySequence_Size(item) != 2) {
@@ -337,8 +344,8 @@ _multidict_extend(MultiDictObject *self, PyObject *args, PyObject *kwds,
     if (args && PyObject_Length(args) > 1)  {
         PyErr_Format(
             PyExc_TypeError,
-            "%s takes at most 1 positional argument (%zd given)",
-            name, PyObject_Length(args), NULL
+            "%s takes from 1 to 2 positional arguments but %zd were given",
+            name, PyObject_Length(args) + 1, NULL
         );
         return -1;
     }
@@ -440,13 +447,31 @@ fail:
 /******************** Base Methods ********************/
 
 static inline PyObject *
-multidict_getall(MultiDictObject *self, PyObject *const *args,
-                 Py_ssize_t nargs, PyObject *kwnames)
+multidict_getall(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *list     = NULL,
              *key      = NULL,
              *_default = NULL;
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *getall_keywords[] = {"key", "default", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O:getall",
+                                     getall_keywords, &key, &_default))
+    {
+        return NULL;
+    }
+#else
     static const char * const _keywords[] = {"key", "default", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"O|O:getall", _keywords, 0};
@@ -470,6 +495,7 @@ multidict_getall(MultiDictObject *self, PyObject *const *args,
 
     _default = args[1];
 skip_optional_pos:
+#endif
 #endif
     list = pair_list_get_all(&self->pairs, key);
 
@@ -486,12 +512,31 @@ skip_optional_pos:
 }
 
 static inline PyObject *
-multidict_getone(MultiDictObject *self, PyObject *const *args,
-                 Py_ssize_t nargs, PyObject *kwnames)
+multidict_getone(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *key      = NULL,
              *_default = NULL;
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *getone_keywords[] = {"key", "default", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O:getone",
+                                     getone_keywords, &key, &_default))
+    {
+        return NULL;
+    }
+
+#else
     static const char * const _keywords[] = {"key", "default", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"O|O:getone", _keywords, 0};
@@ -515,18 +560,37 @@ multidict_getone(MultiDictObject *self, PyObject *const *args,
 
     _default = args[1];
 skip_optional_pos:
+#endif
 #endif
     return _multidict_getone(self, key, _default);
 }
 
 static inline PyObject *
-multidict_get(MultiDictObject *self, PyObject *const *args,
-                 Py_ssize_t nargs, PyObject *kwnames)
+multidict_get(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *key      = NULL,
              *_default = Py_None,
              *ret;
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *getone_keywords[] = {"key", "default", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O:getone",
+                                     getone_keywords, &key, &_default))
+    {
+        return NULL;
+    }
+#else
     static const char * const _keywords[] = {"key", "default", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"O|O:get", _keywords, 0};
@@ -550,6 +614,7 @@ multidict_get(MultiDictObject *self, PyObject *const *args,
 
     _default = args[1];
 skip_optional_pos:
+#endif
 #endif
     ret = _multidict_getone(self, key, _default);
     return ret;
@@ -709,13 +774,13 @@ static inline void
 multidict_tp_dealloc(MultiDictObject *self)
 {
     PyObject_GC_UnTrack(self);
-    Py_TRASHCAN_SAFE_BEGIN(self);
+    Py_TRASHCAN_BEGIN(self, multidict_tp_dealloc)
     if (self->weaklist != NULL) {
         PyObject_ClearWeakRefs((PyObject *)self);
     };
     pair_list_dealloc(&self->pairs);
     Py_TYPE(self)->tp_free((PyObject *)self);
-    Py_TRASHCAN_SAFE_END(self);
+    Py_TRASHCAN_END // there should be no code after this
 }
 
 static inline int
@@ -763,12 +828,29 @@ multidict_tp_init(MultiDictObject *self, PyObject *args, PyObject *kwds)
 }
 
 static inline PyObject *
-multidict_add(MultiDictObject *self, PyObject *const *args,
-              Py_ssize_t nargs, PyObject *kwnames)
+multidict_add(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *key = NULL,
              *val = NULL;
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *kwlist[] = {"key", "value", NULL};
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO:add",
+                                     kwlist, &key, &val))
+    {
+        return NULL;
+    }
+#else
     static const char * const _keywords[] = {"key", "value", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"OO:add", _keywords, 0};
@@ -777,9 +859,12 @@ multidict_add(MultiDictObject *self, PyObject *const *args,
         return NULL;
     }
 #else
-    static _PyArg_Parser _parser = {NULL, _keywords, "add", 0};
+    static _PyArg_Parser _parser = {
+        .keywords = _keywords,
+        .fname = "add",
+        .kwtuple = NULL,
+    };
     PyObject *argsbuf[2];
-
     args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames,
                                  &_parser, 2, 2, 0, argsbuf);
     if (!args) {
@@ -787,6 +872,7 @@ multidict_add(MultiDictObject *self, PyObject *const *args,
     }
     key = args[0];
     val = args[1];
+#endif
 #endif
     if (pair_list_add(&self->pairs, key, val) < 0) {
         return NULL;
@@ -822,12 +908,30 @@ multidict_clear(MultiDictObject *self)
 }
 
 static inline PyObject *
-multidict_setdefault(MultiDictObject *self, PyObject *const *args,
-                     Py_ssize_t nargs, PyObject *kwnames)
+multidict_setdefault(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *key      = NULL,
              *_default = NULL;
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *setdefault_keywords[] = {"key", "default", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O:setdefault",
+                                     setdefault_keywords, &key, &_default))
+    {
+        return NULL;
+    }
+#else
     static const char * const _keywords[] = {"key", "default", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"O|O:setdefault", _keywords, 0};
@@ -852,18 +956,50 @@ multidict_setdefault(MultiDictObject *self, PyObject *const *args,
     _default = args[1];
 
 skip_optional_pos:
+#endif
 #endif
     return pair_list_set_default(&self->pairs, key, _default);
 }
 
 static inline PyObject *
-multidict_popone(MultiDictObject *self, PyObject *const *args,
-                     Py_ssize_t nargs, PyObject *kwnames)
+multidict_popone(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *key      = NULL,
              *_default = NULL,
              *ret_val  = NULL;
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *popone_keywords[] = {"key", "default", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O:popone",
+                                     popone_keywords, &key, &_default))
+    {
+        return NULL;
+    }
+
+    ret_val = pair_list_pop_one(&self->pairs, key);
+
+    if (ret_val == NULL &&
+        PyErr_ExceptionMatches(PyExc_KeyError) &&
+        _default != NULL)
+    {
+        PyErr_Clear();
+        Py_INCREF(_default);
+        return _default;
+    }
+
+    return ret_val;
+#else
     static const char * const _keywords[] = {"key", "default", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"O|O:popone", _keywords, 0};
@@ -901,16 +1037,36 @@ skip_optional_pos:
     }
 
     return ret_val;
+#endif
 }
 
 static inline PyObject *
-multidict_pop(MultiDictObject *self, PyObject *const *args,
-                     Py_ssize_t nargs, PyObject *kwnames)
+multidict_pop(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *key      = NULL,
              *_default = NULL,
              *ret_val  = NULL;
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *pop_keywords[] = {"key", "default", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O:popone",
+                                     pop_keywords, &key, &_default))
+    {
+        return NULL;
+    }
+
+#else
     static const char * const _keywords[] = {"key", "default", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"O|O:pop", _keywords, 0};
@@ -935,6 +1091,7 @@ multidict_pop(MultiDictObject *self, PyObject *const *args,
     _default = args[1];
 
 skip_optional_pos:
+#endif
 #endif
     ret_val = pair_list_pop_one(&self->pairs, key);
 
@@ -951,14 +1108,32 @@ skip_optional_pos:
 }
 
 static inline PyObject *
-multidict_popall(MultiDictObject *self, PyObject *const *args,
-                     Py_ssize_t nargs, PyObject *kwnames)
+multidict_popall(
+    MultiDictObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
     PyObject *key      = NULL,
              *_default = NULL,
              *ret_val  = NULL;
 
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    static char *popall_keywords[] = {"key", "default", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O:popall",
+                                     popall_keywords, &key, &_default))
+    {
+        return NULL;
+    }
+#else
     static const char * const _keywords[] = {"key", "default", NULL};
 #ifdef FASTCALL_OLD
     static _PyArg_Parser _parser = {"O|O:popall", _keywords, 0};
@@ -983,6 +1158,7 @@ multidict_popall(MultiDictObject *self, PyObject *const *args,
     _default = args[1];
 
 skip_optional_pos:
+#endif
 #endif
     ret_val = pair_list_pop_all(&self->pairs, key);
 
@@ -1051,16 +1227,7 @@ PyDoc_STRVAR(multidict_update_doc,
 "Update the dictionary from *other*, overwriting existing keys.");
 
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
 #define multidict_class_getitem Py_GenericAlias
-#else
-static inline PyObject *
-multidict_class_getitem(PyObject *self, PyObject *arg)
-{
-    Py_INCREF(self);
-    return self;
-}
-#endif
 
 
 PyDoc_STRVAR(sizeof__doc__,
@@ -1091,19 +1258,34 @@ static PyMethodDef multidict_methods[] = {
     {
         "getall",
         (PyCFunction)multidict_getall,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_getall_doc
     },
     {
         "getone",
         (PyCFunction)multidict_getone,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_getone_doc
     },
     {
         "get",
         (PyCFunction)multidict_get,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_get_doc
     },
     {
@@ -1127,7 +1309,12 @@ static PyMethodDef multidict_methods[] = {
     {
         "add",
         (PyCFunction)multidict_add,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_add_doc
     },
     {
@@ -1151,25 +1338,45 @@ static PyMethodDef multidict_methods[] = {
     {
         "setdefault",
         (PyCFunction)multidict_setdefault,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_setdefault_doc
     },
     {
         "popone",
         (PyCFunction)multidict_popone,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_popone_doc
     },
     {
         "pop",
         (PyCFunction)multidict_pop,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_pop_doc
     },
     {
         "popall",
         (PyCFunction)multidict_popall,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_popall_doc
     },
     {
@@ -1338,24 +1545,76 @@ multidict_proxy_tp_init(MultiDictProxyObject *self, PyObject *args,
 }
 
 static inline PyObject *
-multidict_proxy_getall(MultiDictProxyObject *self, PyObject *const *args,
-                       Py_ssize_t nargs, PyObject *kwnames)
+multidict_proxy_getall(
+    MultiDictProxyObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
-    return multidict_getall(self->md, args, nargs, kwnames);
+    return multidict_getall(
+        self->md,
+        args,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        kwds
+#else
+        nargs,
+        kwnames
+#endif
+    );
 }
 
 static inline PyObject *
-multidict_proxy_getone(MultiDictProxyObject *self, PyObject *const *args,
-                       Py_ssize_t nargs, PyObject *kwnames)
+multidict_proxy_getone(
+    MultiDictProxyObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
-    return multidict_getone(self->md, args, nargs, kwnames);
+    return multidict_getone(
+        self->md, args,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        kwds
+#else
+        nargs, kwnames
+#endif
+    );
 }
 
 static inline PyObject *
-multidict_proxy_get(MultiDictProxyObject *self, PyObject *const *args,
-                       Py_ssize_t nargs, PyObject *kwnames)
+multidict_proxy_get(
+    MultiDictProxyObject *self,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    PyObject *args,
+    PyObject *kwds
+#else
+    PyObject *const *args,
+    Py_ssize_t nargs,
+    PyObject *kwnames
+#endif
+)
 {
-    return multidict_get(self->md, args, nargs, kwnames);
+    return multidict_get(
+        self->md,
+        args,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        kwds
+#else
+        nargs,
+        kwnames
+#endif
+    );
 }
 
 static inline PyObject *
@@ -1463,19 +1722,34 @@ static PyMethodDef multidict_proxy_methods[] = {
     {
         "getall",
         (PyCFunction)multidict_proxy_getall,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_getall_doc
     },
     {
         "getone",
         (PyCFunction)multidict_proxy_getone,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_getone_doc
     },
     {
         "get",
         (PyCFunction)multidict_proxy_get,
-        METH_FASTCALL | METH_KEYWORDS,
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        METH_VARARGS
+#else
+        METH_FASTCALL
+#endif
+        | METH_KEYWORDS,
         multidict_get_doc
     },
     {
@@ -1655,6 +1929,7 @@ getversion(PyObject *self, PyObject *md)
 static inline void
 module_free(void *m)
 {
+    Py_CLEAR(multidict_str_lower);
     Py_CLEAR(collections_abc_mapping);
     Py_CLEAR(collections_abc_mut_mapping);
     Py_CLEAR(collections_abc_mut_multi_mapping);
@@ -1681,24 +1956,16 @@ static PyModuleDef multidict_module = {
 };
 
 PyMODINIT_FUNC
-PyInit__multidict()
+PyInit__multidict(void)
 {
+    multidict_str_lower = PyUnicode_InternFromString("lower");
+    if (multidict_str_lower == NULL) {
+        goto fail;
+    }
+
     PyObject *module = NULL,
              *reg_func_call_result = NULL;
 
-#define WITH_MOD(NAME)                      \
-    Py_CLEAR(module);                       \
-    module = PyImport_ImportModule(NAME);   \
-    if (module == NULL) {                   \
-        goto fail;                          \
-    }
-
-#define GET_MOD_ATTR(VAR, NAME)                 \
-    VAR = PyObject_GetAttrString(module, NAME); \
-    if (VAR == NULL) {                          \
-        goto fail;                              \
-    }
-
     if (multidict_views_init() < 0) {
         goto fail;
     }
@@ -1719,18 +1986,31 @@ PyInit__multidict()
         goto fail;
     }
 
+#define WITH_MOD(NAME)                      \
+    Py_CLEAR(module);                       \
+    module = PyImport_ImportModule(NAME);   \
+    if (module == NULL) {                   \
+        goto fail;                          \
+    }
+
+#define GET_MOD_ATTR(VAR, NAME)                 \
+    VAR = PyObject_GetAttrString(module, NAME); \
+    if (VAR == NULL) {                          \
+        goto fail;                              \
+    }
+
     WITH_MOD("collections.abc");
     GET_MOD_ATTR(collections_abc_mapping, "Mapping");
 
     WITH_MOD("multidict._abc");
     GET_MOD_ATTR(collections_abc_mut_mapping, "MultiMapping");
-
-    WITH_MOD("multidict._abc");
     GET_MOD_ATTR(collections_abc_mut_multi_mapping, "MutableMultiMapping");
 
     WITH_MOD("multidict._multidict_base");
     GET_MOD_ATTR(repr_func, "_mdrepr");
 
+    Py_CLEAR(module);                       \
+
     /* Register in _abc mappings (CI)MultiDict and (CI)MultiDictProxy */
     reg_func_call_result = PyObject_CallMethod(
         collections_abc_mut_mapping,
@@ -1774,6 +2054,13 @@ PyInit__multidict()
 
     /* Instantiate this module */
     module = PyModule_Create(&multidict_module);
+    if (module == NULL) {
+        goto fail;
+    }
+
+#ifdef Py_GIL_DISABLED
+    PyUnstable_Module_SetGIL(module, Py_MOD_GIL_NOT_USED);
+#endif
 
     Py_INCREF(&istr_type);
     if (PyModule_AddObject(
@@ -1813,6 +2100,7 @@ PyInit__multidict()
     return module;
 
 fail:
+    Py_XDECREF(multidict_str_lower);
     Py_XDECREF(collections_abc_mapping);
     Py_XDECREF(collections_abc_mut_mapping);
     Py_XDECREF(collections_abc_mut_multi_mapping);
