@@ -8,8 +8,7 @@ extern "C" {
 #include <string.h>
 #include <stddef.h>
 #include <stdint.h>
-
-typedef PyObject * (*calc_identity_func)(PyObject *key);
+#include <stdbool.h>
 
 typedef struct pair {
     PyObject  *identity;  // 8
@@ -32,18 +31,14 @@ The embedded buffer intention is to fit the vast majority of possible
 HTTP headers into the buffer without allocating an extra memory block.
 */
 
-#if (PY_VERSION_HEX < 0x03080000)
-#define EMBEDDED_CAPACITY 28
-#else
 #define EMBEDDED_CAPACITY 29
-#endif
 
-typedef struct pair_list {  // 40
-    Py_ssize_t  capacity;   // 8
-    Py_ssize_t  size;       // 8
-    uint64_t  version;      // 8
-    calc_identity_func calc_identity;  // 8
-    pair_t *pairs;          // 8
+typedef struct pair_list {
+    Py_ssize_t capacity;
+    Py_ssize_t size;
+    uint64_t version;
+    bool calc_ci_indentity;
+    pair_t *pairs;
     pair_t buffer[EMBEDDED_CAPACITY];
 } pair_list_t;
 
@@ -111,7 +106,7 @@ ci_key_to_str(PyObject *key)
         return ret;
     }
     if (PyUnicode_Check(key)) {
-        return _PyObject_CallMethodId(key, &PyId_lower, NULL);
+        return PyObject_CallMethodNoArgs(key, multidict_str_lower);
     }
     PyErr_SetString(PyExc_TypeError,
                     "CIMultiDict keys should be either str "
@@ -199,30 +194,38 @@ pair_list_shrink(pair_list_t *list)
 
 
 static inline int
-_pair_list_init(pair_list_t *list, calc_identity_func calc_identity)
+_pair_list_init(pair_list_t *list, bool calc_ci_identity)
 {
+    list->calc_ci_indentity = calc_ci_identity;
     list->pairs = list->buffer;
     list->capacity = EMBEDDED_CAPACITY;
     list->size = 0;
     list->version = NEXT_VERSION();
-    list->calc_identity = calc_identity;
     return 0;
 }
 
 static inline int
 pair_list_init(pair_list_t *list)
 {
-    return _pair_list_init(list, key_to_str);
+    return _pair_list_init(list, /* calc_ci_identity = */ false);
 }
 
 
 static inline int
 ci_pair_list_init(pair_list_t *list)
 {
-    return _pair_list_init(list, ci_key_to_str);
+    return _pair_list_init(list, /* calc_ci_identity = */ true);
 }
 
 
+static inline PyObject *
+pair_list_calc_identity(pair_list_t *list, PyObject *key)
+{
+    if (list->calc_ci_indentity)
+        return ci_key_to_str(key);
+    return key_to_str(key);
+}
+
 static inline void
 pair_list_dealloc(pair_list_t *list)
 {
@@ -304,7 +307,7 @@ pair_list_add(pair_list_t *list,
     PyObject *identity = NULL;
     int ret;
 
-    identity = list->calc_identity(key);
+    identity = pair_list_calc_identity(list, key);
     if (identity == NULL) {
         goto fail;
     }
@@ -412,7 +415,7 @@ pair_list_del(pair_list_t *list, PyObject *key)
     Py_hash_t hash;
     int ret;
 
-    identity = list->calc_identity(key);
+    identity = pair_list_calc_identity(list, key);
     if (identity == NULL) {
         goto fail;
     }
@@ -486,7 +489,11 @@ pair_list_contains(pair_list_t *list, PyObject *key)
     PyObject *identity = NULL;
     int tmp;
 
-    ident = list->calc_identity(key);
+    if (!PyUnicode_Check(key)) {
+        return 0;
+    }
+
+    ident = pair_list_calc_identity(list, key);
     if (ident == NULL) {
         goto fail;
     }
@@ -528,7 +535,7 @@ pair_list_get_one(pair_list_t *list, PyObject *key)
     PyObject *value = NULL;
     int tmp;
 
-    ident = list->calc_identity(key);
+    ident = pair_list_calc_identity(list, key);
     if (ident == NULL) {
         goto fail;
     }
@@ -573,7 +580,7 @@ pair_list_get_all(pair_list_t *list, PyObject *key)
     PyObject *res = NULL;
     int tmp;
 
-    ident = list->calc_identity(key);
+    ident = pair_list_calc_identity(list, key);
     if (ident == NULL) {
         goto fail;
     }
@@ -631,7 +638,7 @@ pair_list_set_default(pair_list_t *list, PyObject *key, PyObject *value)
     PyObject *value2 = NULL;
     int tmp;
 
-    ident = list->calc_identity(key);
+    ident = pair_list_calc_identity(list, key);
     if (ident == NULL) {
         goto fail;
     }
@@ -680,7 +687,7 @@ pair_list_pop_one(pair_list_t *list, PyObject *key)
     int tmp;
     PyObject *ident = NULL;
 
-    ident = list->calc_identity(key);
+    ident = pair_list_calc_identity(list, key);
     if (ident == NULL) {
         goto fail;
     }
@@ -730,7 +737,7 @@ pair_list_pop_all(pair_list_t *list, PyObject *key)
     PyObject *res = NULL;
     PyObject *ident = NULL;
 
-    ident = list->calc_identity(key);
+    ident = pair_list_calc_identity(list, key);
     if (ident == NULL) {
         goto fail;
     }
@@ -826,7 +833,7 @@ pair_list_replace(pair_list_t *list, PyObject * key, PyObject *value)
     PyObject *identity = NULL;
     Py_hash_t hash;
 
-    identity = list->calc_identity(key);
+    identity = pair_list_calc_identity(list, key);
     if (identity == NULL) {
         goto fail;
     }
@@ -905,13 +912,18 @@ _pair_list_post_update(pair_list_t *list, PyObject* used_keys, Py_ssize_t pos)
 
     for (; pos < list->size; pos++) {
         pair = pair_list_get(list, pos);
-        tmp = PyDict_GetItem(used_keys, pair->identity);
-        if (tmp == NULL) {
+        int status = PyDict_GetItemRef(used_keys, pair->identity, &tmp);
+        if (status == -1) {
+            // exception set
+            return -1;
+        }
+        else if (status == 0) {
             // not found
             continue;
         }
 
         num = PyLong_AsSsize_t(tmp);
+        Py_DECREF(tmp);
         if (num == -1) {
             if (!PyErr_Occurred()) {
                 PyErr_SetString(PyExc_RuntimeError, "invalid internal state");
@@ -944,12 +956,18 @@ _pair_list_update(pair_list_t *list, PyObject *key,
     int found;
     int ident_cmp_res;
 
-    item = PyDict_GetItem(used_keys, identity);
-    if (item == NULL) {
+    int status = PyDict_GetItemRef(used_keys, identity, &item);
+    if (status == -1) {
+        // exception set
+        return -1;
+    }
+    else if (status == 0) {
+        // not found
         pos = 0;
     }
     else {
         pos = PyLong_AsSsize_t(item);
+        Py_DECREF(item);
         if (pos == -1) {
             if (!PyErr_Occurred()) {
                 PyErr_SetString(PyExc_RuntimeError, "invalid internal state");
@@ -1076,18 +1094,28 @@ pair_list_update_from_seq(pair_list_t *list, PyObject *seq)
         }
 
         // Convert item to sequence, and verify length 2.
+#ifdef Py_GIL_DISABLED
+        if (!PySequence_Check(item)) {
+#else
         fast = PySequence_Fast(item, "");
         if (fast == NULL) {
             if (PyErr_ExceptionMatches(PyExc_TypeError)) {
+#endif
                 PyErr_Format(PyExc_TypeError,
                              "multidict cannot convert sequence element #%zd"
                              " to a sequence",
                              i);
+#ifndef Py_GIL_DISABLED
             }
+#endif
             goto fail_1;
         }
 
+#ifdef Py_GIL_DISABLED
+        n = PySequence_Size(item);
+#else
         n = PySequence_Fast_GET_SIZE(fast);
+#endif
         if (n != 2) {
             PyErr_Format(PyExc_ValueError,
                          "multidict update sequence element #%zd "
@@ -1096,12 +1124,29 @@ pair_list_update_from_seq(pair_list_t *list, PyObject *seq)
             goto fail_1;
         }
 
+#ifdef Py_GIL_DISABLED
+        key = PySequence_ITEM(item, 0);
+        if (key == NULL) {
+            PyErr_Format(PyExc_ValueError,
+                         "multidict update sequence element #%zd's "
+                         "key could not be fetched", i);
+            goto fail_1;
+        }
+        value = PySequence_ITEM(item, 1);
+        if (value == NULL) {
+            PyErr_Format(PyExc_ValueError,
+                         "multidict update sequence element #%zd's "
+                         "value could not be fetched", i);
+            goto fail_1;
+        }
+#else
         key = PySequence_Fast_GET_ITEM(fast, 0);
         value = PySequence_Fast_GET_ITEM(fast, 1);
         Py_INCREF(key);
         Py_INCREF(value);
+#endif
 
-        identity = list->calc_identity(key);
+        identity = pair_list_calc_identity(list, key);
         if (identity == NULL) {
             goto fail_1;
         }
@@ -1117,7 +1162,9 @@ pair_list_update_from_seq(pair_list_t *list, PyObject *seq)
 
         Py_DECREF(key);
         Py_DECREF(value);
+#ifndef Py_GIL_DISABLED
         Py_DECREF(fast);
+#endif
         Py_DECREF(item);
         Py_DECREF(identity);
     }
@@ -1148,11 +1195,11 @@ pair_list_eq_to_mapping(pair_list_t *list, PyObject *other)
 {
     PyObject *key = NULL;
     PyObject *avalue = NULL;
-    PyObject *bvalue = NULL;
+    PyObject *bvalue;
 
-    Py_ssize_t pos;
+    Py_ssize_t pos, other_len;
 
-    int cmp;
+    int eq;
 
     if (!PyMapping_Check(other)) {
         PyErr_Format(PyExc_TypeError,
@@ -1161,7 +1208,11 @@ pair_list_eq_to_mapping(pair_list_t *list, PyObject *other)
         return -1;
     }
 
-    if (pair_list_len(list) != PyMapping_Length(other)) {
+    other_len = PyMapping_Size(other);
+    if (other_len < 0) {
+        return -1;
+    }
+    if (pair_list_len(list) != other_len) {
         return 0;
     }
 
@@ -1169,21 +1220,18 @@ pair_list_eq_to_mapping(pair_list_t *list, PyObject *other)
     while (pair_list_next(list, &pos, NULL, &key, &avalue)) {
         bvalue = PyObject_GetItem(other, key);
         if (bvalue == NULL) {
-            PyErr_Clear();
-            return 0;
-        }
-
-        cmp = PyObject_RichCompareBool(avalue, bvalue, Py_EQ);
-        Py_DECREF(bvalue);
-
-        if (cmp < 0) {
+            if (PyErr_ExceptionMatches(PyExc_KeyError)) {
+                PyErr_Clear();
+                return 0;
+            }
             return -1;
         }
-        else if (cmp > 0) {
-            continue;
-        }
-        else {
-            return 0;
+
+        eq = PyObject_RichCompareBool(avalue, bvalue, Py_EQ);
+        Py_DECREF(bvalue);
+
+        if (eq <= 0) {
+            return eq;
         }
     }
 
